<?php

namespace FiloBlu\Refilo\Model\Exporter;

use FiloBlu\Refilo\Model\Adapter\Mongo;
use FiloBlu\Refilo\Model\Config\ConfigProviderInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\DataObjectFactory;
use Magento\Store\Api\Data\StoreInterface;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\StoreManagerInterface;
use MongoDB\Driver\Exception\Exception;
use stdClass;

/**
 * Class Config
 * @package FiloBlu\Refilo\Model\Exporter
 */
class Config
{
    /**
     * @var StoreManagerInterface
     */
    private $storeManager;
    /**
     * @var ScopeConfigInterface
     */
    private $scopeConfig;
    /**
     * @var Mongo
     */
    private $mongo;
    /**
     * @var array
     */
    private $providers;
    /**
     * @var DataObjectFactory
     */
    private $dataObjectFactory;

    /**
     * Config constructor.
     * @param StoreManagerInterface $storeManager
     * @param ScopeConfigInterface $scopeConfig
     * @param DataObjectFactory $dataObjectFactory
     * @param Mongo $mongo
     * @param array $providers
     */
    public function __construct(
        StoreManagerInterface $storeManager,
        ScopeConfigInterface $scopeConfig,
        DataObjectFactory $dataObjectFactory,
        Mongo $mongo,
        array $providers = []
    ) {
        $this->storeManager = $storeManager;
        $this->scopeConfig = $scopeConfig;
        $this->mongo = $mongo;
        $this->providers = $providers;
        $this->dataObjectFactory = $dataObjectFactory;
    }

    /**
     *
     */
    public function exportConfig()
    {
        $stores = $this->storeManager->getStores(false, true);

        $bulkData = [];

        foreach ($stores as $code => $store) {
            $configurations = $this->scopeConfig->getValue(null, ScopeInterface::SCOPE_STORE, $code);
            $bulkData[] = $this->process($configurations, $store);
        }

        $configurations = $this->scopeConfig->getValue(null);
        $bulkData[] = $this->process($configurations);

        try {
            $this->mongo->update(
                '_id',
                $bulkData,
                'configs',
                true,
                $this->scopeConfig->getValue('mongo/general/mongo_general_database')
            );

            $this->mongo->ensureIndex(['metacode', 'metadata.code'], 'configs');
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    /**
     * @param array $configurations
     * @param StoreInterface| null$store
     * @return array
     */
    protected function process(array $configurations, $store =null)
    {
        $data = [];
        /** @var ConfigProviderInterface $provider */
        foreach ($this->providers as $provider) {
            $result = $provider->getConfigs($this->dataObjectFactory->create(['data' => $configurations]), $store);
            $data = array_merge_recursive($data, $result);
        }

        return $this->encodeData($data, $store ? $store->getCode() : null);
    }

    /**
     * @param array $data
     * @param string|null $storeCode
     * @return array
     */
    protected function encodeData(array $data, $storeCode = null)
    {
        return [
            '_id'      => $storeCode ? "country_{$storeCode}" : 'default_config',
            'metacode' => $storeCode ? 'store' : 'default',
            'metadata' => $data
        ];
    }

    /**
     * @deprecated
     * @param $config
     * @return array
     */
    public function getListConfig($config)
    {
        $convertedConfig = [];
        $unserializedConfig = unserialize($config);
        foreach ($unserializedConfig as $config) {
            $convertedConfig[] = (object)$config;
        }

        return $convertedConfig;
    }

    /**
     * @deprecated
     * @param $config
     * @return array
     */
    public function getCcBrands($config)
    {
        $convertedConfig = [];
        $unserializedConfig = unserialize($config);
        foreach ($unserializedConfig as $config) {
            $convertedConfig[] = $config['code'];
        }

        return $convertedConfig;
    }
}
